// Heyo / TMI_js by Sarimoko.com
// ================================
// Hello World Chatbot for Twitch IRC using TMI.js!
// --------------------------------
// View README.md for .env setup!!!
// --------------------------------
require('dotenv').config();
const tmi = require('tmi.js');

console.log(process.env.API_HOST);

const client = new tmi.Client({
  options: { debug: true },
  connection: {
    secure: true,
    reconnect: true
  },
  identity: {
    username: process.env.TTV_BOT_USERNAME,
    password: process.env.TTV_BOT_OAUTH
  },
  channels: [ process.env.TTV_CHANNELS ]
});

client.connect();

// RNG User
const rp = require('request-promise');

function getChatters(channelName, _attemptCount = 0) {
    return rp({
        uri: `https://tmi.twitch.tv/group/user/sarimoko/chatters`,
        json: true
    })
    .then(data => {
        return Object.entries(data.chatters)
            .reduce((p, [ type, list ]) => p.concat(list.map(name => {
                if(name === channelName) type = 'broadcaster';
                return { name, type };
            })), []);
    })
    .catch(err => {
        if(_attemptCount < 3) {
            return getChatters(channelName, _attemptCount + 1);
        }
        throw err;
    })
}

function getRandomChatter(channelName, opts = {}) {
    let {
        onlyViewers = false,
        noBroadcaster = false,
        skipList = []
    } = opts;
    return getChatters(channelName)
    .then(data => {
        let chatters = data
            .filter(({ name, type }) =>
                !(
                    (onlyViewers && type !== 'viewers') ||
                    (noBroadcaster && type === 'broadcaster') ||
                    skipList.includes(name)
                )
            );
        return chatters.length === 0 ?
            null :
            chatters[Math.floor(Math.random() * chatters.length)];
    });
} // RNG END

// Chatbot Commands
client.on('message', (channel, tags, message, self) => { // START | MSG
    if(self) return; // Bot ignores itself
    const badges = tags.badges || {}; // Scan Badges
    const isBroadcaster = badges.broadcaster; // Define Streamer
    const isMod = badges.moderator; // Define Mod
    const isModUp = isBroadcaster || isMod; // Permission Merge = Mod+Streamer
    const isSub = badges.subscriber || badges.founder; // Define Subs
    const botUserState = client.userstate[channel]; // MOD Status Check
    const amMod = botUserState !== undefined && botUserState.mod === true; // Define Mod Status

    // CMD | CHAT COMMANDS | Start
    // CMD | RNGUser
    if(command === 'rnguser') {
        // Get a random user but skip the user requesting a random user
        getRandomChatter(channel, { skipList: [ tags.username ] })
        .then(user => {
            if(user === null) {
                client.say(channel, `Sorry ${tags.username}, there was no one to choose from!`);
            }
            else {
                let { name, type } = user;
                client.say(channel, `${tags.username} randomly chooses "${name}" as the best ${type}!`);
            }
        })
        .catch(err => console.log('[ERR]', err));
    }
    // CMD | Color
    if(command === 'color') {
      // Change your username color. Color must be in hex (#000000) or one of the following: Blue, BlueViolet, CadetBlue, Chocolate, Coral, DodgerBlue, Firebrick, GoldenRod, Green, HotPink, OrangeRed, Red, SeaGreen, SpringGreen, YellowGreen.
      client.say(channel, `/color ${args.join(' ')}`);
      client.say(channel, `/me @${tags.username} has changed my color to ${args.join(' ')}`);
    }

    if(isSub) { // SUB | Start
      if(command === 'hi' ||
      command === 'hello') {
        client.say(channel, `!kappagen sarimoKO sarimoNERD Bleep-Bloop!`);
      }
    } // SUB | End

    if(isModUp) { // MOD Start
    // Shoutout
      if(command === 'so' ||
      command === 'shoutout') {
        client.say(channel, `/me Roll the clip! Smash @${args.join(' ')}'s follow button at: https://twitch.tv/${args.join(' ')} Bleep-Bloop!`);
      }
    }

}); // END | MSG

// TTV Re-Active
// ==================================
// client.on("action", (channel, userstate, message, self) => {
//     // Don't listen to my own messages..
//     if (self) return;
//     console.log('LOG: Action ');
// });
client.on("anongiftpaidupgrade", (channel, username, userstate) => {
    console.log('TTV Alert: Anon2PaidSub...'); // VERY specific... Anon Gift Sub => Paid Re-Sub
});
client.on("ban", (channel, username, reason, userstate) => {
    console.log('TTV Alert: User Banned...'); // BYE FELICIA!
});
// client.on("chat", (channel, userstate, message, self) => {
//     // Don't listen to my own messages..
//     if (self) return;
//     // Do your stuff.
//       console.log('LOG: Chat');
// });
client.on("cheer", (channel, userstate, message) => {
    console.log('TTV Alert: Bits Cheered...'); // Get Bits via Cheer then React
});
client.on("clearchat", (channel) => {
    console.log('TTV Alert: Chat Cleared...'); // Chat gets cleared by Mod/Streamer then this Reaction
});
client.on("connected", (address, port) => {
      console.log('LOG: Connected ');
});
client.on("connecting", (address, port) => {
      console.log('LOG: Connecting ');
});
client.on("disconnected", (reason) => {
      console.log('LOG: Dis-Connected ');
});
client.on("emoteonly", (channel, enabled) => {
      console.log('CHAT MODE: Emotes Only ');
});
// client.on("emotesets", (sets, obj) => {
//     // Here are the emotes I can use:
//       console.log('Emote Sets: IDK WHAT TO DO WITH THESE ');
//     console.log(obj);
// });
client.on("followersonly", (channel, enabled, length) => {
      console.log('CHAT MOD: Followers only enabled!');
});
client.on("giftpaidupgrade", (channel, username, sender, userstate) => {
    console.log('TTV Alert: Anon2PaidSub...'); // Huh? Tier 1 => Tier2?
});
client.on("hosted", (channel, username, viewers, autohost) => {
      console.log('LOG: TYVM! For the host!');
});
client.on("hosting", (channel, target, viewers) => {
    console.log('TTV Alert: Incoming Host...'); // Get Hosted React
});
client.on("join", (channel, username, self) => {
    console.log('TTV Alert: User joined IRC...');// User joins IRC **WARNING: aggro/annoying
});
client.on("logon", () => {
      console.log('LOG: Logon ');
});
// client.on("message", (channel, userstate, message, self) => {
//     // Don't listen to my own messages..
//     if (self) return;
// 
//     // Handle different message types..
//     switch(userstate["message-type"]) {
//         case "action":
//             // This is an action message..
//             break;
//         case "chat":
//             // This is a chat message..
//             break;
//         case "whisper":
//             // This is a whisper..
//             break;
//         default:
//             // Something else ?
//             break;
//     }
// });
client.on("messagedeleted", (channel, username, deletedMessage, userstate) => {
      console.log('LOG: Msg deleted ');
});
client.on("mod", (channel, username) => {
    // Modded.
      console.log('LOG: Modded ');
});
client.on("mods", (channel, mods) => {
    // List
      console.log('LOG: Mods ');
});
client.on("notice", (channel, msgid, message) => {
      console.log('LOG: Notice ');
});
client.on("part", (channel, username, self) => {
    console.log('TTV Alert: User left IRC...');// User leaves IRC **WARNING: aggro/annoying
});
//client.on("ping", () => {});
//client.on("pong", (latency) => {});
// client.on("r9kbeta", (channel, enabled) => {
//       console.log('r9kbeta? ');
// });
client.on("raided", (channel, username, viewers) => {
    console.log('TTV Alert: Incoming Raid...');// Get Raided React
});
// client.on("raw_message", (messageCloned, message) => {
//     console.log(message.raw);
// });
client.on("reconnect", () => {
    // Do your stuff.
      console.log('Re-Connecting ');
});
client.on("resub", function (channel, username, months, message, userstate, methods) {
      console.log('Re-Sub ');
});
// client.on("roomstate", (channel, state) => {
//       console.log('LOG: Roomstate ');
// });
client.on("serverchange", (channel) => {
      console.log('LOG: Server Change ');
});
client.on("slowmode", (channel, enabled, length) => {
      console.log('CHAT MODE: Slow Mode enabled! ');
      client.say(channel, `CHAT-MODE: Slow`);
});
client.on("subgift", (channel, username, streakMonths, recipient, methods, userstate) => {
    let senderCount = ~~userstate["msg-param-sender-count"];
    console.log('TTV Alert: Sub Gifted...');// Gift Sub React
});
client.on("submysterygift", (channel, username, numbOfSubs, methods, userstate) => {
    let senderCount = ~~userstate["msg-param-sender-count"];
    console.log('TTV Alert: Sub Gifted Anonomously...');// Anon Sub React
});
client.on("resub", function (channel, username, months, message, userstate, methods) {
    console.log('TTV Alert: Sub Re-Subbed...');// Re-Sub React
});
client.on("subscribers", (channel, enabled) => {
    client.say(channel, `CHAT-MODE: Subscribers only!`);
});
client.on("subscription", function (channel, username, method, message, userstate) {
    console.log('TTV Alert: 1st Sub...'); // Either first or just General Sub
});
client.on("timeout", (channel, username, reason, duration, userstate) => {
    client.say(channel, `I know this only frustrates you more, but please take this time to cooldown.`);
});
client.on("unhost", (channel, viewers) => {
    client.say(channel, `/me @Sarimoko has turned if hosting, please standby for further instruction!`);
});
client.on("unmod", (channel, username) => {
    client.say(channel, `Oof, modship`);
});
client.on("vips", (channel, vips) => {
    client.say(channel, `@Sarimoko is fortunate to have many great viewers but these VIPs are a few specifically to thank!`);
});
client.on("whisper", (from, userstate, message, self) => {
    if (self) return;
    console.log('TTV Alert: DM/Whisper recieved...'); // When your bot gets a DM
});